#include "ipofml.h"
#include <QVBoxLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QVariant>
#include <QVariantList>
#include <QPixmap>
#include <QTimer>
#include <QStringList>
#include <QPainter>
#include <QFileDialog>
#include <QListWidgetItem>
#include "math.h"
#include "mtimer.h"

#define CLSID_DDC1100 "{3F5D1418-3DE0-437A-A834-28E5314D326E}"

#ifdef USE_WLP_DMD
#define loadToDMD(a, b)	trigger(b)
#endif

IPOFML::IPOFML(QWidget *parent, Qt::WindowFlags flags)
	: QMainWindow(parent, flags)
{
	ui.setupUi(this);

	m_nActType = ACTION_NONE;
	m_nCurRow = m_nCurColumn = 0;
	m_nIPMLDMDMemBuf = 0;

	initUVSource();
	initStage();
	initDMD();

	setTabWidget();
}

IPOFML::~IPOFML()
{
}

/*=================================================*/
/* UV Source Control                               */
/*=================================================*/
void IPOFML::initUVSource()
{
	m_pLC8 = new LC8();
	m_pLC8Ctrl = new LC8Control();

	m_pLC8Ctrl->connect(LC8_COMPORT_NAME);
	
	connect(m_pLC8Ctrl, SIGNAL(signalConnected()), m_pLC8Ctrl, SLOT(queryIntensity()));
	connect(m_pLC8Ctrl, SIGNAL(signalIntensity(int)), this, SLOT(slotUVIntensity(int)));
}

void IPOFML::slotUVIntensity(int percent)
{
	m_pLC8->setIntensity(percent);
}

void IPOFML::slotUVShutterClosed()
{
	int movingTime = (m_pInterMovingTime->text()).toDouble() * 1000;
	if(movingTime > 0) QTimer::singleShot(movingTime, this, SLOT(slotOnInterMovingTimer()));
	else QTimer::singleShot(0, this, SLOT(slotOnInterMovingTimer()));
}

/*=================================================*/
/* Stage Control                                   */
/*=================================================*/
void IPOFML::initStage()
{
	m_pTango = new CTango();

	m_pTango->ConnectSimple(1, STAGE_COMPORT_NAME, STAGE_COMPORT_BAUDRATE, false);
}

/*=================================================*/
/* DMD Control                                     */
/*=================================================*/
void IPOFML::initDMD()
{
	m_pDDC1100 = new DMD(CLSID_DDC1100, this);
	connect(m_pDDC1100, SIGNAL(signalDMDCleared()), this, SLOT(slotDMDCleared()));
}

/*=================================================*/
/* User Interface                                  */
/*=================================================*/
void IPOFML::setTabWidget()
{
	intensityUpBtn = new QPushButton("Up");
	intensityDownBtn = new QPushButton("Down");
	intensityUpBtn->setFixedWidth(40);
	intensityDownBtn->setFixedWidth(40);

	connect(intensityUpBtn, SIGNAL(pressed()), m_pLC8Ctrl, SLOT(intensityUp()));
	connect(intensityDownBtn, SIGNAL(pressed()), m_pLC8Ctrl, SLOT(intensityDown()));
	connect(intensityUpBtn, SIGNAL(released()), m_pLC8Ctrl, SLOT(intensityStop()));
	connect(intensityDownBtn, SIGNAL(released()), m_pLC8Ctrl, SLOT(intensityStop()));

	m_pShutterTime = new QLineEdit();
	m_pShutterTime->setFixedWidth(70);
	m_pShutterTime->setAlignment(Qt::AlignRight);
	m_pShutterTime->setText(QString("%1").arg(DEFAULT_SHUTTER_TIME));
	m_pLC8->setShutterTime(DEFAULT_SHUTTER_TIME);
	
	connect(m_pShutterTime, SIGNAL(textChanged(const QString &)), m_pLC8, SLOT(setShutterTime(const QString &)));

	QHBoxLayout *uvHBox = new QHBoxLayout();
	uvHBox->addStretch();
	uvHBox->addWidget(intensityUpBtn);
	uvHBox->addWidget(intensityDownBtn);
	uvHBox->addStretch();
	uvHBox->addWidget(m_pShutterTime);
	uvHBox->addStretch();

	QVBoxLayout *uvVBox = new QVBoxLayout();
	uvVBox->addWidget(m_pLC8, 0, Qt::AlignCenter);
	uvVBox->addLayout(uvHBox);

	QGroupBox *uvGroup = new QGroupBox();
	uvGroup->setTitle("UV Source");
	uvGroup->setLayout(uvVBox);

	ui.verticalLayout_2->addWidget(uvGroup);	
	
	QLabel *disXLabel = new QLabel("X-axis Step(mm) :");
	m_pDistanceX = new QLineEdit();
	m_pDistanceX->setFixedWidth(70);
	m_pDistanceX->setAlignment(Qt::AlignRight);
	m_pDistanceX->setText(QString("%1").arg(DEFAULT_STEP_X));
	QLabel *disYLabel = new QLabel("Y-axis Step(mm) :");
	m_pDistanceY = new QLineEdit();
	m_pDistanceY->setFixedWidth(70);
	m_pDistanceY->setAlignment(Qt::AlignRight);
	m_pDistanceY->setText(QString("%1").arg(DEFAULT_STEP_Y));

	QLabel *numRowLabel = new QLabel("Row :");
	m_pNumRow = new QLineEdit();
	m_pNumRow->setFixedWidth(70);
	m_pNumRow->setAlignment(Qt::AlignRight);
	m_pNumRow->setText(QString("%1").arg(DEFAULT_NUM_ROW));
	QLabel *numColumnLabel = new QLabel("Column :");
	m_pNumColumn = new QLineEdit();
	m_pNumColumn->setFixedWidth(70);
	m_pNumColumn->setAlignment(Qt::AlignRight);
	m_pNumColumn->setText(QString("%1").arg(DEFAULT_NUM_COLUMN));

	QLabel *interMovingTimeLabel = new QLabel("Inter-moving time(s) :");
	m_pInterMovingTime = new QLineEdit();
	m_pInterMovingTime->setFixedWidth(70);
	m_pInterMovingTime->setAlignment(Qt::AlignRight);
	m_pInterMovingTime->setText(QString("%1").arg(DEFAULT_INTERMOVING_TIME));

	m_pAct2 = new QPushButton("Preview");
	connect(m_pAct2, SIGNAL(clicked()), this, SLOT(slotAct2Clicked()));

	m_pActionStopBtn = new QPushButton(tr("Stop"));
	m_pActionStopBtn->setEnabled(false);
	connect(m_pActionStopBtn, SIGNAL(clicked()), this, SLOT(slotActionStopBtnClicked()));

	QGridLayout *stageGrid = new QGridLayout();
	stageGrid->addWidget(disXLabel, 0, 0, Qt::AlignRight);
	stageGrid->addWidget(m_pDistanceX, 0, 1, Qt::AlignLeft);
	stageGrid->addWidget(disYLabel, 0, 2, Qt::AlignRight);
	stageGrid->addWidget(m_pDistanceY, 0, 3, Qt::AlignLeft);
	stageGrid->addWidget(numRowLabel, 1, 0, Qt::AlignRight);
	stageGrid->addWidget(m_pNumRow, 1, 1, Qt::AlignLeft);
	stageGrid->addWidget(numColumnLabel, 1, 2, Qt::AlignRight);
	stageGrid->addWidget(m_pNumColumn, 1, 3, Qt::AlignLeft);
	stageGrid->addWidget(interMovingTimeLabel, 2, 0, Qt::AlignRight);
	stageGrid->addWidget(m_pInterMovingTime, 2, 1, Qt::AlignLeft);
	stageGrid->addWidget(m_pAct2, 2, 2, 1, 1, Qt::AlignCenter);
	stageGrid->addWidget(m_pActionStopBtn, 2, 3, 1, 1, Qt::AlignCenter);

	QGroupBox *stageGroup = new QGroupBox();
	stageGroup->setTitle("Stage");
	stageGroup->setLayout(stageGrid);

	ui.verticalLayout_2->addWidget(stageGroup);	

#ifdef USE_WLP_DMD
	m_pMaskNSFlip = new QCheckBox(QString("NS Flip"));
	m_pMaskWEFlip = new QCheckBox(QString("WE Flip"));

	QHBoxLayout *flipHLayout = new QHBoxLayout();
	flipHLayout->addWidget(m_pMaskNSFlip);
	flipHLayout->addWidget(m_pMaskWEFlip);
	flipHLayout->addStretch();

	QGroupBox *dmdGroup = new QGroupBox();
	dmdGroup->setTitle("DMD");
	dmdGroup->setLayout(flipHLayout);

	ui.verticalLayout_2->addWidget(dmdGroup);	
#endif


	ui.tabWidget->setTabText(0, "List");
	ui.tabWidget->setTabText(1, "IP-ML");
	ui.tabWidget->setTabText(2, "Setting");
	ui.tabWidget->setCurrentIndex(1);


	// List Tab
	m_pListPreviewMask = new QLabel();
	m_pListPreviewMask->setFixedSize(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT);
	m_pListPreviewMask->setPixmap(QPixmap(QString::fromUtf8(":/IPOFML/Resources/black.bmp")).scaled(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT));

	m_pMaskTableWidget = new QTableWidget(this);
	QStringList maskListHeader;
	maskListHeader << tr("Mask") << tr("Time (ms)");
	m_pMaskTableWidget->setColumnCount(maskListHeader.count());
	m_pMaskTableWidget->setHorizontalHeaderLabels(maskListHeader);
	m_pMaskTableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_pMaskTableWidget->setFixedWidth(420);
	m_pMaskTableWidget->setColumnWidth(0, 300);
	m_pMaskTableWidget->setColumnWidth(1, 120);

	m_pAddListBtn = new QPushButton(tr("Add"));
	m_pDeleteListBtn = new QPushButton(tr("Del"));
	m_pClearListBtn = new QPushButton(tr("Clear"));
	m_pUpListBtn = new QPushButton(tr("Up"));
	m_pDownListBtn = new QPushButton(tr("Down"));

	connect(m_pMaskTableWidget, SIGNAL(currentCellChanged(int, int, int, int)), this, SLOT(slotListCurrentRowChanged(int, int, int, int)));
	connect(m_pAddListBtn, SIGNAL(clicked()), this, SLOT(slotAddListBtnClicked()));
	connect(m_pDeleteListBtn, SIGNAL(clicked()), this, SLOT(slotDeleteListBtnClicked()));
	connect(m_pClearListBtn, SIGNAL(clicked()), this, SLOT(slotClearListBtnClicked()));
	connect(m_pUpListBtn, SIGNAL(clicked()), this, SLOT(slotUpListBtnClicked()));
	connect(m_pDownListBtn, SIGNAL(clicked()), this, SLOT(slotDownListBtnClicked()));

	QHBoxLayout *btnHBox = new QHBoxLayout();
	btnHBox->addWidget(m_pAddListBtn);
	btnHBox->addWidget(m_pDeleteListBtn);
	btnHBox->addWidget(m_pClearListBtn);
	btnHBox->addWidget(m_pUpListBtn);
	btnHBox->addWidget(m_pDownListBtn);

	m_pListOnOneSpotCBox = new QCheckBox(tr("List on one spot"), this);
	m_pListOnOneSpotCBox->setChecked(false);

	m_pAct1 = new QPushButton("Action");
	m_pAct1->setEnabled(false);
	connect(m_pAct1, SIGNAL(clicked()), this, SLOT(slotAct1Clicked()));
	
	QVBoxLayout *tab1Layout = new QVBoxLayout();	
	tab1Layout->addWidget(m_pListPreviewMask, 0, Qt::AlignCenter);
	tab1Layout->addWidget(m_pMaskTableWidget);
	tab1Layout->addLayout(btnHBox);
	tab1Layout->addStretch();
	tab1Layout->addWidget(m_pListOnOneSpotCBox);
	tab1Layout->addWidget(m_pAct1);

	ui.tab->setLayout(tab1Layout);

	
	// IP-ML Tab
	m_pIPMLPreviewMask = new QLabel();
	m_pIPMLPreviewMask->setFixedSize(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT);
	m_pIPMLPreviewMask->setPixmap(QPixmap(QString::fromUtf8(":/IPOFML/Resources/black.bmp")).scaled(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT));

	m_pIPMLMask = new QLineEdit();
	m_pIPMLMask->setReadOnly(true);
	
	m_pIPMLMaskBtn = new QPushButton(tr("open"));
	connect(m_pIPMLMaskBtn, SIGNAL(clicked()), this, SLOT(slotIPMLMaskBtnClicked()));

	QGridLayout *ipmlMaskGrid = new QGridLayout();
	ipmlMaskGrid->addWidget(m_pIPMLPreviewMask, 0, 0, 1, 2, Qt::AlignCenter);
	ipmlMaskGrid->addWidget(m_pIPMLMask, 1, 0, 1, 1);
	ipmlMaskGrid->addWidget(m_pIPMLMaskBtn, 1, 1, 1, 1);

	m_pAct3 = new QPushButton("Action");
	m_pAct3->setEnabled(false);
	connect(m_pAct3, SIGNAL(clicked()), this, SLOT(slotAct3Clicked()));

	QVBoxLayout *tab2Layout = new QVBoxLayout();
	tab2Layout->addLayout(ipmlMaskGrid);
	tab2Layout->addStretch();
	tab2Layout->addWidget(m_pAct3);

	ui.tab_2->setLayout(tab2Layout);


	// Setting Tab
	QLabel *portLabel = new QLabel("UV Source :");
	m_pUVPort = new QComboBox();
	m_pUVPort->setFixedWidth(70);
	QString portName;
	for(int i=1; i<=MAX_COMPORT_NUM; i++)
	{
		portName = QString("COM%1").arg(i);
		m_pUVPort->addItem(portName);

		if(LC8_COMPORT_NAME == portName) m_pUVPort->setCurrentIndex(i-1);
	}
	QPushButton *portBtn = new QPushButton("Set");
	portBtn->setFixedWidth(40);

	// Motorized Stage
	QLabel *stagePortLabel = new QLabel("Stage :");
	m_pStagePort = new QComboBox();
	m_pStagePort->setFixedWidth(70);
	for(int i=1; i<=MAX_COMPORT_NUM; i++)
	{
		portName = QString("COM%1").arg(i);
		m_pStagePort->addItem(portName);

		if(STAGE_COMPORT_NAME == portName) m_pStagePort->setCurrentIndex(i-1);
	}

	QPushButton *stagePortBtn = new QPushButton("Set");
	stagePortBtn->setFixedWidth(40);

	QGridLayout *portLayout = new QGridLayout();
	portLayout->addWidget(portLabel, 0, 0, Qt::AlignRight);
	portLayout->addWidget(m_pUVPort, 0, 1, Qt::AlignLeft);
	portLayout->addWidget(portBtn, 0, 2, Qt::AlignLeft);
	portLayout->addWidget(stagePortLabel, 1, 0, Qt::AlignRight);
	portLayout->addWidget(m_pStagePort, 1, 1, Qt::AlignLeft);
	portLayout->addWidget(stagePortBtn, 1, 2, Qt::AlignLeft);

	QGroupBox *portGroup = new QGroupBox();
	portGroup->setTitle("Port");
	portGroup->setLayout(portLayout);

	connect(portBtn, SIGNAL(clicked()), this, SLOT(slotSetUVComPort()));
	connect(stagePortBtn, SIGNAL(clicked()), this, SLOT(slotSetStageComPort()));

	QVBoxLayout *tab3Layout = new QVBoxLayout();
	tab3Layout->addWidget(portGroup);
	tab3Layout->addStretch();

	ui.tab_3->setLayout(tab3Layout);
}

void IPOFML::actionUIEnabled(bool enable)
{
	intensityUpBtn->setEnabled(enable);
	intensityDownBtn->setEnabled(enable);
	m_pShutterTime->setEnabled(enable);
	m_pDistanceX->setEnabled(enable);
	m_pDistanceY->setEnabled(enable);
	m_pNumColumn->setEnabled(enable);
	m_pNumRow->setEnabled(enable);
	m_pInterMovingTime->setEnabled(enable);
	m_pAct2->setEnabled(enable);
	ui.tabWidget->setEnabled(enable);
	m_pListOnOneSpotCBox->setEnabled(enable);
}

void IPOFML::slotIPMLMaskBtnClicked()
{
	QString file = QFileDialog::getOpenFileName(this, tr("Open Mask"), "", tr("Image (*.bmp)"));

    if(file.isEmpty() != true)
    {
#ifdef USE_WLP_DMD
#else
		m_nIPMLDMDMemBuf = m_pDDC1100->loadToMemBuffer(file);
#endif
		m_pIPMLPreviewMask->setPixmap(QPixmap(file).scaled(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT));
		m_pIPMLMask->setText(file);

		m_pAct3->setEnabled(true);
    }
}

void IPOFML::slotAddListBtnClicked()
{
	QStringList files = QFileDialog::getOpenFileNames(this, tr("Open Mask"), "", tr("Image (*.bmp)"));
	files.sort();

	if(files.isEmpty()) return;	

	int bufNum, row, i, count = files.count();
	QTableWidgetItem *itemMask, *itemExposure;

	for(i=0; i<count; i++)
	{
#ifdef USE_WLP_DMD
#else
		bufNum = m_pDDC1100->loadToMemBuffer(files.at(i));
#endif

		itemMask = new QTableWidgetItem(files.at(i));
		itemExposure = new QTableWidgetItem("100");
#ifdef USE_WLP_DMD
		itemMask->setData(Qt::UserRole, QVariant(files.at(i)));
#else
		itemMask->setData(Qt::UserRole, QVariant(bufNum));
#endif

		row = m_pMaskTableWidget->rowCount();
		m_pMaskTableWidget->setRowCount(row+1);
		m_pMaskTableWidget->setItem(row, 0, itemMask);
		m_pMaskTableWidget->setItem(row, 1, itemExposure);
	}
	
	if(m_pMaskTableWidget->rowCount() > 0) m_pAct1->setEnabled(true);
}

void IPOFML::slotDeleteListBtnClicked()
{
	if(m_pMaskTableWidget->rowCount() > 0)
	{
		int currentRow = m_pMaskTableWidget->currentRow();
		if(currentRow >= 0)
		{
			QTableWidgetItem *item = m_pMaskTableWidget->item(currentRow, 0);
#ifdef USE_WLP_DMD
#else
			m_pDDC1100->unloadFromMemBuffer(item->data(Qt::UserRole).toInt());
#endif
			m_pMaskTableWidget->removeRow(currentRow);	
		}
	}

	if(m_pMaskTableWidget->rowCount() == 0) m_pAct1->setEnabled(false);
}

void IPOFML::slotClearListBtnClicked()
{
	int i;
	int count = m_pMaskTableWidget->rowCount();
	for(i=0; i<count; i++)
	{
		QTableWidgetItem *item = m_pMaskTableWidget->item(0, 0);
#ifdef USE_WLP_DMD
#else
		m_pDDC1100->unloadFromMemBuffer(item->data(Qt::UserRole).toInt());
#endif
		m_pMaskTableWidget->removeRow(0);	
	}

	m_pAct1->setEnabled(false);
}

void IPOFML::slotUpListBtnClicked()
{
	int currentRow = m_pMaskTableWidget->currentRow();

	if(currentRow > 0)
	{
		QTableWidgetItem *curItemMask = m_pMaskTableWidget->takeItem(currentRow, 0);
		QTableWidgetItem *curItemExposure = m_pMaskTableWidget->takeItem(currentRow, 1);

		QTableWidgetItem *upperItemMask = m_pMaskTableWidget->takeItem(currentRow-1, 0);
		QTableWidgetItem *upperItemExposure = m_pMaskTableWidget->takeItem(currentRow-1, 1);

		m_pMaskTableWidget->setItem(currentRow, 0, upperItemMask);
		m_pMaskTableWidget->setItem(currentRow, 1, upperItemExposure);

		m_pMaskTableWidget->setItem(currentRow-1, 0, curItemMask);
		m_pMaskTableWidget->setItem(currentRow-1, 1, curItemExposure);

		m_pMaskTableWidget->setCurrentItem(curItemMask);
	}
}

void IPOFML::slotDownListBtnClicked()
{
	int currentRow = m_pMaskTableWidget->currentRow();
	int rowCount = m_pMaskTableWidget->rowCount();

	if(currentRow>=0 && currentRow<rowCount-1)
	{
		QTableWidgetItem *curItemMask = m_pMaskTableWidget->takeItem(currentRow, 0);
		QTableWidgetItem *curItemExposure = m_pMaskTableWidget->takeItem(currentRow, 1);

		QTableWidgetItem *lowerItemMask = m_pMaskTableWidget->takeItem(currentRow+1, 0);
		QTableWidgetItem *lowerItemExposure = m_pMaskTableWidget->takeItem(currentRow+1, 1);

		m_pMaskTableWidget->setItem(currentRow, 0, lowerItemMask);
		m_pMaskTableWidget->setItem(currentRow, 1, lowerItemExposure);

		m_pMaskTableWidget->setItem(currentRow+1, 0, curItemMask);
		m_pMaskTableWidget->setItem(currentRow+1, 1, curItemExposure);

		m_pMaskTableWidget->setCurrentItem(curItemMask);
	}
}

void IPOFML::slotListCurrentRowChanged(int currentRow, int currentColumn, int previousRow, int previousColumn)
{
	if(currentRow >= 0)
	{
		QTableWidgetItem *item = m_pMaskTableWidget->item(currentRow, 0);	
		m_pListPreviewMask->setPixmap(QPixmap(item->text()).scaled(PREVIEW_MASK_WIDTH, PREVIEW_MASK_HEIGHT));
	}
}

void IPOFML::slotSetUVComPort()
{
	m_pLC8Ctrl->disconnect();
	m_pLC8Ctrl->connect(m_pUVPort->currentText());
}

void IPOFML::slotSetStageComPort()
{
	m_pTango->Disconnect();
	m_pTango->ConnectSimple(1, (m_pStagePort->currentText()).toLatin1().data(), STAGE_COMPORT_BAUDRATE, false);
}

void IPOFML::slotAct1Clicked()
{
	actionUIEnabled(false);

	m_nActType = ACTION_1;
	m_nCurRow = m_nCurColumn = 0;

	m_pTango->SetPos(0.0, 0.0, 0.0, 0.0);

	listIndex = 0;

	QTableWidgetItem *itemMask = m_pMaskTableWidget->item(0, 0);
	m_pMaskTableWidget->setCurrentItem(itemMask);

	QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(0, 1);
	int msec = (itemExposure->text()).toInt();

	m_pLC8Ctrl->shutterOpen();

#ifdef USE_WLP_DMD
	QStringList fileNames;
	QTableWidgetItem *item;
	for(int i=0; i<m_pMaskTableWidget->rowCount(); i++)
	{
		item = m_pMaskTableWidget->item(i, 0);
		fileNames.append((item->data(Qt::UserRole)).toString());
	}
	m_pDDC1100->loadFiles(fileNames, m_pMaskNSFlip->isChecked(), m_pMaskWEFlip->isChecked());
	m_pDDC1100->startDMD();
#endif
	m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
}

void IPOFML::slotAct2Clicked()
{
	actionUIEnabled(false);

	m_nActType = ACTION_2;
	m_nCurRow = m_nCurColumn = 0;

	m_pTango->SetPos(0.0, 0.0, 0.0, 0.0);

	int movingTime = (m_pInterMovingTime->text()).toDouble() * 1000;
	if(movingTime == 0)
	{
		slotOnAct2Timer();
	}
	else
	{
		QTimer::singleShot(movingTime, this, SLOT(slotOnAct2Timer()));
	}
}

void IPOFML::slotAct3Clicked()
{
	actionUIEnabled(false);

	m_nActType = ACTION_3;
	m_nCurRow = m_nCurColumn = 0;

	m_pTango->SetPos(0.0, 0.0, 0.0, 0.0);

	int msec = (m_pShutterTime->text()).toDouble() * 1000;
	m_pLC8Ctrl->shutterOpen();

#ifdef USE_WLP_DMD
	QStringList file;
	file.append(m_pIPMLMask->text());
	m_pDDC1100->loadFiles(file, m_pMaskNSFlip->isChecked(), m_pMaskWEFlip->isChecked());
	m_pDDC1100->startDMD();
#endif
	m_pDDC1100->loadToDMD(m_nIPMLDMDMemBuf, msec);
}

void IPOFML::slotOnAct2Timer()
{
	int numRow = (m_pNumRow->text()).toInt();
	int numColumn = (m_pNumColumn->text()).toInt();

	double stepX = (m_pDistanceX->text()).toDouble();
	double stepY = (m_pDistanceY->text()).toDouble();

	int movingTime = (m_pInterMovingTime->text()).toDouble() * 1000;

	m_nCurColumn++;
	if(m_nCurColumn < numColumn)
	{
		if(m_nCurRow % 2) m_pTango->MoveRel(stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
		else m_pTango->MoveRel(-stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

		QTimer::singleShot(movingTime, this, SLOT(slotOnAct2Timer()));
	}
	else
	{
		m_nCurRow++;
		m_nCurColumn = 0;
		if(m_nCurRow < numRow)
		{
			m_pTango->MoveRel(0.0, -stepY, 0.0, 0.0, STAGE_WAIT_FLAG);

			QTimer::singleShot(movingTime, this, SLOT(slotOnAct2Timer()));
		}
		else
		{
			m_pTango->MoveAbs(0.0, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
			actionUIEnabled(true);
		}
	}
}

void IPOFML::slotDMDCleared()
{
	if(m_nActType==ACTION_1 && m_pListOnOneSpotCBox->isChecked())
	{
		listIndex++;
		if(listIndex < m_pMaskTableWidget->rowCount())
		{
			QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(listIndex, 1);
			int msec = (itemExposure->text()).toInt();
			QTableWidgetItem *itemMask = m_pMaskTableWidget->item(listIndex, 0);	
			m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
			m_pMaskTableWidget->setCurrentItem(itemMask);
		}
		else
		{
			listIndex = 0;

			int movingTime = (m_pInterMovingTime->text()).toDouble() * 1000;
			if(movingTime > 0) QTimer::singleShot(movingTime, this, SLOT(slotOnInterMovingTimerByDMD()));
			else QTimer::singleShot(0, this, SLOT(slotOnInterMovingTimerByDMD()));
		}
	}
	else
	{
		int movingTime = (m_pInterMovingTime->text()).toDouble() * 1000;
		if(movingTime > 0) QTimer::singleShot(movingTime, this, SLOT(slotOnInterMovingTimerByDMD()));
		else QTimer::singleShot(0, this, SLOT(slotOnInterMovingTimerByDMD()));
	}
}

void IPOFML::slotOnInterMovingTimerByDMD()
{
	int numRow = (m_pNumRow->text()).toInt();
	int numColumn = (m_pNumColumn->text()).toInt();

	double stepX = (m_pDistanceX->text()).toDouble();
	double stepY = (m_pDistanceY->text()).toDouble();

	int msec = (m_pShutterTime->text()).toDouble() * 1000;

	switch(m_nActType)
	{
		case ACTION_1:
			if(m_pListOnOneSpotCBox->isChecked())
			{
				m_nCurColumn++;
				if(m_nCurColumn < numColumn)
				{
					if(m_nCurRow % 2) m_pTango->MoveRel(stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
					else m_pTango->MoveRel(-stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
					
					QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(listIndex, 1);
					msec = (itemExposure->text()).toInt();
					QTableWidgetItem *itemMask = m_pMaskTableWidget->item(listIndex, 0);	
					m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
					m_pMaskTableWidget->setCurrentItem(itemMask);
				}
				else
				{
					m_nCurRow++;
					m_nCurColumn = 0;
					if(m_nCurRow < numRow)
					{
						m_pTango->MoveRel(0.0, -stepY, 0.0, 0.0, STAGE_WAIT_FLAG);

						QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(listIndex, 1);
						msec = (itemExposure->text()).toInt();
						QTableWidgetItem *itemMask = m_pMaskTableWidget->item(listIndex, 0);	
						m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
						m_pMaskTableWidget->setCurrentItem(itemMask);
					}
					else
					{
						m_pLC8Ctrl->shutterClose();
						m_pTango->MoveAbs(0.0, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

						actionUIEnabled(true);

#ifdef USE_WLP_DMD
						m_pDDC1100->stopDMD();
#endif
					}
				}
			}
			else
			{
				listIndex++;
				m_nCurColumn++;

				if(m_nCurColumn < numColumn)
				{
					if(m_nCurRow % 2) m_pTango->MoveRel(stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
					else m_pTango->MoveRel(-stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
					
					listIndex = listIndex % (m_pMaskTableWidget->rowCount());
					QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(listIndex, 1);
					msec = (itemExposure->text()).toInt();
					QTableWidgetItem *itemMask = m_pMaskTableWidget->item(listIndex, 0);	
					m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
					m_pMaskTableWidget->setCurrentItem(itemMask);
				}
				else
				{
					m_nCurRow++;
					m_nCurColumn = 0;
					if(m_nCurRow < numRow)
					{
						m_pTango->MoveRel(0.0, -stepY, 0.0, 0.0, STAGE_WAIT_FLAG);

						listIndex = listIndex % (m_pMaskTableWidget->rowCount());
						QTableWidgetItem *itemExposure = m_pMaskTableWidget->item(listIndex, 1);
						msec = (itemExposure->text()).toInt();
						QTableWidgetItem *itemMask = m_pMaskTableWidget->item(listIndex, 0);	
						m_pDDC1100->loadToDMD((itemMask->data(Qt::UserRole)).toInt(), msec);
						m_pMaskTableWidget->setCurrentItem(itemMask);
					}
					else
					{
						m_pLC8Ctrl->shutterClose();
						m_pTango->MoveAbs(0.0, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

						actionUIEnabled(true);
#ifdef USE_WLP_DMD
						m_pDDC1100->stopDMD();
#endif
					}
				}
			}
		break;

		case ACTION_2:
		break;

		case ACTION_3:
			m_nCurColumn++;
			if(m_nCurColumn < numColumn)
			{		
				if(m_nCurRow % 2) m_pTango->MoveRel(stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);
				else m_pTango->MoveRel(-stepX, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

				m_pDDC1100->loadToDMD(m_nIPMLDMDMemBuf, msec);
			}
			else
			{
				m_nCurRow++;
				m_nCurColumn = 0;
				if(m_nCurRow < numRow)
				{
					m_pTango->MoveRel(0.0, -stepY, 0.0, 0.0, STAGE_WAIT_FLAG);

					m_pDDC1100->loadToDMD(m_nIPMLDMDMemBuf, msec);
				}
				else
				{
					m_pLC8Ctrl->shutterClose();
					m_pTango->MoveAbs(0.0, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

					actionUIEnabled(true);

#ifdef USE_WLP_DMD
					m_pDDC1100->stopDMD();
#endif
				}
			}
		break;	
	}
}

void IPOFML::slotActionStopBtnClicked()
{
	m_pLC8Ctrl->shutterClose();
	m_pDDC1100->clear();
	m_pTango->MoveAbs(0.0, 0.0, 0.0, 0.0, STAGE_WAIT_FLAG);

	actionUIEnabled(true);
	m_pActionStopBtn->setEnabled(false);

	m_nActType = ACTION_NONE;
}
